/****************************************************************************\
*                                                                            *
*  RAINBOW.CPP                                                               *
*  RAINBOWU.CPP                                                              *
*                                                                            *
*  This program demonstrates color palette cycling.                          *
*                                                                            *
\****************************************************************************/

#include <vcl\vcl.h>
#pragma hdrstop

#include "RainbowU.h"
//---------------------------------------------------------------------------
#pragma resource "*.dfm"
TForm1 *Form1;
//---------------------------------------------------------------------------
__fastcall TForm1::TForm1(TComponent* Owner)
	: TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TForm1::FormActivate(TObject *Sender)
{
   fg_realize(hpal);
   Invalidate();
}
//---------------------------------------------------------------------------
void __fastcall TForm1::FormCreate(TObject *Sender)
{
   // create the logical palette
   hdc = GetDC(Form1->Handle);
   fg_setdc(hdc);
   fill_color_palette();
   hpal = fg_logpal(10,24,RGBvalues);
   fg_realize(hpal);

   // create a 640x480 virtual buffer
   fg_vbinit();
   hvb = fg_vballoc(640,480);
   fg_vbopen(hvb);
   fg_vbcolors();

   // construct a crude image of a rainbow
   fg_setcolor(255);
   fg_fillpage();
   fg_setclip(0,639,0,300);
   fg_move(320,300);
   xlen = 240;
   ylen = 120;
   for (color = 10; color < 34; color++)
   {
      fg_setcolor(color);
      fg_ellipsef(xlen,ylen);
      xlen -= 4;
      ylen -= 3;
   }
   fg_setcolor(255);
   fg_ellipsef(xlen,ylen);
   fg_setclip(0,639,0,479);

   // starting index into the array of color values
   start = 0;

   Application->OnActivate = OnActivate;
}
//---------------------------------------------------------------------------
void __fastcall TForm1::FormPaint(TObject *Sender)
{
   fg_vbscale(0,fg_getmaxx(),0,fg_getmaxy(),0,cxClient-1,0,cyClient-1);
}
//---------------------------------------------------------------------------
void __fastcall TForm1::FormResize(TObject *Sender)
{
   cxClient = ClientWidth;
   cyClient = ClientHeight;
   Invalidate();
}
//---------------------------------------------------------------------------
void __fastcall TForm1::FormDestroy(TObject *Sender)
{
   fg_vbclose();
   fg_vbfree(hvb);
   fg_vbfin();
   DeleteObject(hpal);
   ReleaseDC(Form1->Handle,hdc);
}
//---------------------------------------------------------------------------
void __fastcall TForm1::Timer1Timer(TObject *Sender)
{
   if (GetActiveWindow() == Form1->Handle)
   {
      start = (start + 3) % 72;
      fg_setdacs(10,24,&RGBvalues[start]);
      if (fg_colors() > 8)
         fg_vbscale(0,fg_getmaxx(),0,fg_getmaxy(),0,cxClient-1,0,cyClient-1);
   }
}

/****************************************************************************\
*                                                                            *
*  fill_color_palette                                                        *
*                                                                            *
*  Set up the colors for the application's logical palette in the RGBvalues  *
*  array. The logical palette will contain 24 non-system colors (indices 10  *
*  to 33) defining the initial RGB values for the colors being cycled.       *
*                                                                            *
*  Note that we store two identical sets of 24 RGB triplets in RGBvalues. We *
*  can then perform color cycling without having to worry about wrapping to  *
*  the start of the array because the index pointing to the starting RGB     *
*  triplet never extends beyond the first set of 24 RGB triplets.            *
*                                                                            *
\****************************************************************************/

void TForm1::fill_color_palette()
{
   static BYTE colors[] = {
      182,182,255, 198,182,255, 218,182,255, 234,182,255, 255,182,255,
      255,182,234, 255,182,218, 255,182,198, 255,182,182, 255,198,182,
      255,218,182, 255,234,182, 255,255,182, 234,255,182, 218,255,182,
      198,255,182, 182,255,182, 182,255,198, 182,255,218, 182,255,234,
      182,255,255, 182,234,255, 182,218,255, 182,198,255};

   // set up two identical sets of the 24 colors in the RGBvalues array
   memcpy(RGBvalues,colors,24*3);
   memcpy(&RGBvalues[24*3],colors,24*3);
}
